# Simple Class Browser

require 'fltk'

class ClassList < FLTK::HoldBrowser
  attr_accessor :method_list

  def initialize(reader,ml, *args)
    super(*args)

    @method_list = ml
    @source_reader = reader

    callback{|widget,data|
      scope = widget.data(widget.value)
      if( scope )
	@method_list.show_defs(scope)
      end
    }

    show_classes
  end

  def show_classes
    clear
    @source_reader.scopes.keys.sort.each{|name|
      if( name == "" )
	add("(main)",name)
      else
	ty = @source_reader.scopes[name].ty
	add(name + " [#{ty}]",name)
      end
    }
  end
end

class MethodList < FLTK::HoldBrowser
  attr_accessor :source_viewer

  def initialize(reader, sv, *args)
    super(*args)

    @scope = nil
    @source_viewer = sv
    @source_reader = reader

    callback{|widget,data|
      method_name = widget.data(widget.value)
      if( method_name )
	@source_viewer.show_source(@scope,method_name)
      end
    }
  end

  def show_defs(scope)
    @scope = scope
    clear
    @source_reader.scopes[scope].defs.keys.sort.each{|name|
      add(name,name)
    }
  end
end

class SourceViewer < FLTK::MultilineOutput
  def initialize(reader, *args)
    @source_reader = reader
    super(*args)
  end

  def show_source(scope, method_name)
    value(@source_reader.scopes[scope].defs[method_name])
  end
end

class SourceReader
  attr_reader :scopes

  def initialize(file = nil)
    init_scope
    if( file )
	parse(file)
    end
  end

  def init_scope
    @cur_scope = []
    @scopes    = {}
    push_scope("", "initial")
  end

  def scope_name
    @cur_scope.join("::")
  end

  def push_scope(name, ty = "class")
    @cur_scope.push(name)
    @scopes[scope_name] ||= Struct.new(nil,:ty,:includes,:parents,:defs).new(ty,[],[],{})
  end

  def pop_scope
    @cur_scope.pop
  end

  def add_def(name,src)
    @scopes[scope_name].defs[name] = src
  end

  def add_include(name)
    @scopes[scope_name].includes.push(name)
  end

  def add_parent(name)
    @scopes[scope_name].parents.push(name)
  end

  def parse(file)
    if( file.is_a?(String) )
      File.open(file, "r"){|f|
	return parse(f)
      }
    end

    cur_scope = []
    while( line = file.gets )
      case line.gsub(/(#.+)|(".+")|('.+')|(\/.+\/)/, "")
      when /^=begin/
	while( line = file.gets )
	  if( line =~ /^=end/ )
	    break
	  end
	end
      when /^module\s+(\w+)/, /\Wmodule\s+(\w+)/
	push_scope($1, "module")
	if( line =~ /\Wend\W/ )
	  pop_scope
	end
      when /^class\s(\w+)(\s+<\s+(\w+))?/, /\Wclass\s(\w+)(\s+<\s+(\w+))?/
	push_scope($1, "class")
	if( $3 )
	  add_parent($3)
	end
	if( line =~ /\Wend\W/ )
	  pop_scope
	end
      when /^include\W+([\w\s,]+)/, /\Winclude\W+([\w\s,]+)/
	add_include($1)
      when /^extend\W+([\w\s,]+)/, /\Wextend\W+([\w\s,]+)/
	add_parent($1)
      when /\s+(if|begin|case|while|for|do|until|unless)\W/, /^(if|begin|case|while|for|do|until|unless)\W/
	if( line !~ /\Wend\W/ && line !~ /\S+\s+if\W/ )
	  push_scope("", "control")
	end
      when /([A-Z](\w+))(\s*)=(\s*)(\S+)/
	src = [line]
	nest = 0
	in_str = false
	line.each_byte{|c|
	  case c.chr
	  when "\"", "\'"
	    in_str = !in_str
	  when "[", "{", "("
	    if( !in_str )
	      nest += 1
	    end
	  when "]", "}", ")"
	    if( !in_str )
	      nest -= 1
	    end
	  end
	}
	while( nest > 0 )
	  line = file.gets
	  src.push(line)
	  line.each_byte{|c|
	    case c.chr
	    when "\"", "\'"
	      in_str = !in_str
	    when "[", "{", "("
	      if( !in_str )
		nest += 1
	      end
	    when "]", "}", ")"
	      if( !in_str )
		nest -= 1
	      end
	    end
	  }
	end
	add_def($1,src.join(""))
      when /^alias\W+?(\w+),?\W+?(\w+)/, /\Walias\W+?(\w+),?\W+?(\w+)/
	add_def($1, line)
      when /^def\s(\w+)\.(\S+)/, /^def\s(\S+)/, /\Wdef\s(\w+)\.(\S+)/, /\Wdef\s(\S+)/
	if( $2 )
	  name = "#{$1}.#{$2}"
	else
	  name = $1
	end
	name.gsub!(/\(.*$/, "")

	src = [line]
	nest = 0
	if( line !~ /\Wend\W/ )
	  while( content = file.gets )
	    src.push(content)
	    case content.gsub(/(#.*)|(".+")|('.+')|(\/.+\/)/, "")
	    when /\W(def|if|begin|case|while|for|do|until|unless)\W/, /^(def|if|begin|case|while|for|do|until|unless)\W/
	      if( content !~ /\Wend\W/ && content !~ /\S+\s+if\W/)
		nest += 1
	      end
	    when /^end\W/, /\Wend\W/
	      if( nest == 0 )
		break
	      else
		nest -= 1
	      end
	    end
	  end
	end
	add_def(name, src.join)
      when /^end\W/, /\s+end\W/
	pop_scope
      end
    end
    delete_invalid_scopes
  end

  def delete_invalid_scopes
    new_scope = {}
    @scopes.select{|scope,info|
      case scope
      when /::$/
      else
	new_scope[scope] = info
      end
    }
    @scopes = new_scope
  end
end

LIST_WIDTH = 200
LIST_HEIGHT = 300
MENU_HEIGHT = 20

VIEW_WIDTH = LIST_WIDTH * 2

WIN_WIDTH = LIST_WIDTH * 2 + VIEW_WIDTH
WIN_HEIGHT = LIST_HEIGHT + MENU_HEIGHT

FLTK::Window.new(WIN_WIDTH, WIN_HEIGHT, "Class Browser"){|win|
  FLTK::Tile.new(0,MENU_HEIGHT,WIN_WIDTH,LIST_HEIGHT){|tile|
    $reader = SourceReader.new
    $viewer = SourceViewer.new($reader,
			       LIST_WIDTH*2,MENU_HEIGHT,VIEW_WIDTH,LIST_HEIGHT)
    $method_list = MethodList.new($reader,$viewer,
				  LIST_WIDTH,MENU_HEIGHT,LIST_WIDTH,LIST_HEIGHT)
    $class_list = ClassList.new($reader,$method_list,
				0,MENU_HEIGHT,LIST_WIDTH,LIST_HEIGHT)
  }

  m = FLTK::MenuBar.new(0, 0, WIN_WIDTH, MENU_HEIGHT)
  m.add("File/Open", nil){
    fname = FLTK.file_chooser("Open")
    if( fname )
      $reader.parse(fname)
      $class_list.show_classes
    end
  }
  m.add("File/Clear", nil){
    $reader.init_scope
    $class_list.show_classes
  }
  m.add("File/Quit", nil){ exit(0) }

  win.resizable(win)
  win.show
}

if( ARGV[0] )
  $reader.parse(ARGV[0])
  $class_list.show_classes
end

FLTK.run
