# This program is a part of the Ruby/FLTK tests.
# Author:: Jeremy Henty <mailto:jeremy_henty@users.sourceforge.net>
# Copyright:: Copyright (c) 2005 Jeremy Henty 
# License:: LGPL

=begin

Show a window containing boxes of all types.

Query the FLTK module dynamically for the box/frame constants.

Two-column layout.  A box with no corresponding frame type spans both
columns.  Otherwise the box goes in the left column and the
corresponding frame goes in the right column.

Box types NO_BOX and FLAT_BOX are special.  Display them at the top.
Raise an error if either is missing.

=end

require 'fltk'

# special box roots 
boxes_special = [ "NO", ]

# hash the box types by the root of their name and whether they are a
# box or a frame
box_pattern = /^((.*)_(BOX|FRAME))$/
class Boxes < Struct.new(:BOX,:FRAME)
  def initialize ; super ; end
end
boxes = Hash.new{|hash,key,| hash[key] = Boxes.new }
FLTK.constants.each{|name|
  next unless match = box_pattern.match(name)
  box, root, type, = *match.captures
  boxes[root][type] = box
}

# list the box roots and move the special ones to the head 
box_roots = boxes.keys
boxes_special.each{|name|
  raise "missing special box constant %s" % [ name, ] unless
    boxes[name]
  box_roots.delete(name)
}
box_roots = boxes_special + box_roots.sort 

# build the GUI 

# layout 
row_count = box_roots.size

# dimensions 
w  = 200 # column width 
mx =   5 # column margin 
h  =  30 # row height 
my =   5 # row margin 

# derived dimensions 
dx = w+mx   # horizontal skip between columns 
w2 = 2*w+mx # width of a double-column cell 
x0 = mx     # x-coordinate of the first column 
x1 = x0+dx  # x-coordinate of the second column 
dy = h+my   # vertical skip between rows 
win_w = (w+mx)*2+mx     # width of the main window 
win_h = dy*row_count+my # height of the main window 

# the left column is the box, the right column is the frame 
x_types = [
  [ x0, :BOX,   ], 
  [ x1, :FRAME, ], 
]

# create the window and boxes 
FLTK::Window.new(win_w,win_h){|win|
  win.label("Box Types")
  y = my
  proc{|box_new|
    box_roots.each{|root|
      box_set = boxes[root]
      if box_set.FRAME
        x_types.each{|x_,type_,|
          box_new.call(box_set[type_],x_,w)
        }
      else
        box_new.call(box_set.BOX,x0,w2)
      end
      y += dy 
    }
  }.call proc{|box_type,x,w_|
    box = FLTK::Box.new(x,y,w_,h,box_type)
    box.box(FLTK.const_get(box_type))
    box.color(FLTK::WHITE) 
  }
  win.show
}

FLTK.run
