# This program is a part of the Ruby/FLTK tests.
# Author:: Jeremy Henty <mailto:jeremy_henty@users.sourceforge.net>
# Copyright:: Copyright (c) 2005 Jeremy Henty 
# License:: LGPL

=begin

Show a window with a box for each available cursor.  Mousing over the
box selects its cursor.

Query the FLTK module dynamically for the cursor constants.

=end

require 'fltk'

# get and hash the cursor constants 
cursor_pattern = /^CURSOR_(.*)$/
cursors = Hash.new
FLTK::Const.constants.each{|name|
  next unless match = cursor_pattern.match(name)
  key, = *match.captures
  cursors[key] = FLTK::Const.const_get(name)
}

# check the special cursors exist and move them to the front of the
# list
keys_special = [
  "DEFAULT", 
  "NONE", 
]
missing_special_key_format = 
  "missing special key: %s"
keys = cursors.keys.sort
keys_special.each{|key|
  raise missing_special_key_format % [ key, ] unless
    cursors.key?(key)
  keys.delete(key)
}
keys = keys_special + keys

# layout 
columns = 4
rows = cursors.size / columns
rows += 1 if
  cursors.size % columns != 0

# sizes 
cw = 100 # column width 
rh =  50 # row height 
w = columns*cw
h = rows*rh

# build the GUI 

class Cursor_Box < FLTK::Box
  # a box that sets the window's cursor when the mouse enters 
  def initialize(win,cursor,*args)
    @win    = win
    @cursor = cursor
    super(*args)
  end
  def handle(e)
    case e
    when FLTK::ENTER
      @win.cursor(@cursor)
      return 1
    end
  end
end

class Cursor_Default_Box < Cursor_Box
  # a box that resets the window's cursor to the default when the
  # mouse enters
  def initialize(win,*args)
    super(win,FLTK::CURSOR_DEFAULT,*args)
  end
end

class Cursor_Named_Box < Cursor_Box
  # a box that displays the name of cursor and sets the window's
  # cursor to the named cursor when the mouse enters
  def initialize(win,cursors,key,*args)
    super(win,cursors[key],*args)
    box(FLTK::DOWN_BOX)
    label(key)
  end
end

FLTK::Window.new(w,h,"Cursors"){|win|
  keys.each_with_index{|key,index|
    column = index % columns
    row    = index / columns
    x = column*cw
    y = row*rh
    Cursor_Named_Box.new(win,cursors,key,x,y,cw,rh)
  }
  unless keys.size % columns == 0
    column = keys.size % columns
    row    = keys.size / columns
    x = column*cw
    y = row*rh
    Cursor_Default_Box.new(win,x,y,(columns-column)*cw,rh) 
  end
  win.show
}

FLTK.run
